from ._ysrc import YsrcValidator
from ._yboundssrc import YboundssrcValidator
from ._ybounds import YboundsValidator
from ._yaxis import YAxisValidator
from ._y import YValidator
from ._xysrc import XysrcValidator
from ._xy import XyValidator
from ._xsrc import XsrcValidator
from ._xboundssrc import XboundssrcValidator
from ._xbounds import XboundsValidator
from ._xaxis import XAxisValidator
from ._x import XValidator
from ._visible import VisibleValidator
from ._uid import UidValidator
from ._textsrc import TextsrcValidator
from ._text import TextValidator
from ._stream import StreamValidator
from ._showlegend import ShowlegendValidator
from ._selectedpoints import SelectedpointsValidator
from ._opacity import OpacityValidator
from ._name import NameValidator
from ._marker import MarkerValidator
from ._legendgroup import LegendgroupValidator
from ._indicessrc import IndicessrcValidator
from ._indices import IndicesValidator
from ._idssrc import IdssrcValidator
from ._ids import IdsValidator
from ._hoverlabel import HoverlabelValidator
from ._hoverinfosrc import HoverinfosrcValidator
from ._hoverinfo import HoverinfoValidator
from ._customdatasrc import CustomdatasrcValidator
from ._customdata import CustomdataValidator
