from ._width import WidthValidator
from ._reversescale import ReversescaleValidator
from ._dash import DashValidator
from ._colorsrc import ColorsrcValidator
from ._colorscale import ColorscaleValidator
from ._color import ColorValidator
from ._cmin import CminValidator
from ._cmax import CmaxValidator
from ._cauto import CautoValidator
from ._autocolorscale import AutocolorscaleValidator
